"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron = require("electron");
const AppConfigApi_1 = require("../shared/config/AppConfigApi");
const LogRendererApi_1 = require("../shared/Log/LogRendererApi");
const AppPreferencesApi_1 = require("../shared/preferences/AppPreferencesApi");
// Set up Preferences API
const preferences = new AppPreferencesApi_1.AppPreferencesApi();
preferences.initialize();
// Set up Config API
const config = new AppConfigApi_1.AppConfigApi();
config.initialize();
//
const log = new LogRendererApi_1.LogRendererApi();
log.bindListeners();
/**
 * Object with functions that bridge between this and the Main processes
 * (Note: This is mostly a left-over from when "node integration" was disabled.
 *        It might be a good idea to move this to the Renderer?)
 */
window.External = Object.freeze({
    /** @inheritDoc */
    platform: electron.remote.process.platform + '',
    /** @inheritDoc */
    minimize() {
        const currentWindow = electron.remote.getCurrentWindow();
        currentWindow.minimize();
    },
    /** @inheritDoc */
    maximize() {
        const currentWindow = electron.remote.getCurrentWindow();
        if (currentWindow.isMaximized()) {
            currentWindow.unmaximize();
        }
        else {
            currentWindow.maximize();
        }
    },
    /** @inheritDoc */
    close() {
        const currentWindow = electron.remote.getCurrentWindow();
        currentWindow.close();
    },
    /** @inheritDoc */
    restart() {
        electron.remote.app.relaunch();
        electron.remote.app.quit();
    },
    /** @inheritDoc */
    showOpenDialog(options, callback) {
        // (Slicing a "remote object" array will make a local copy of it - i think)
        if (callback) {
            // (Returns undefined if a callback is passed)
            electron.remote.dialog.showOpenDialog(options, (filePaths, bookmarks) => {
                callback(filePaths && filePaths.slice(), bookmarks && bookmarks.slice());
            });
        }
        else {
            // (Returns either undefined or string[] if no callback is passed)
            const val = electron.remote.dialog.showOpenDialog(options);
            return val && val.slice();
        }
    },
    /** @inheritDoc */
    toggleDevtools() {
        electron.remote.getCurrentWindow().webContents.toggleDevTools();
    },
    /** @inheritDoc */
    preferences,
    /** @inheritDoc */
    config,
    /** @inheritDoc */
    log,
});
//# sourceMappingURL=MainWindowPreload.js.map