"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = require("child_process");
const electron_1 = require("electron");
const util = require("util");
const execFile = util.promisify(child_process.execFile);
/**
 * Check if an application is installed
 * @param binaryName The command you would use the run an application command
 * @param argument An argument to pass the command. This argument should not cause any side effects. By default --version
 */
async function isInstalled(binaryName, argument = '--version') {
    try {
        await execFile(binaryName, [argument]);
    }
    catch (e) {
        return false;
    }
    return true;
}
exports.isInstalled = isInstalled;
/**
 * If Electron is in development mode (or in release mode)
 * (This is copied straight out of the npm package 'electron-is-dev')
 */
exports.isDev = (function () {
    const getFromEnv = parseInt(process.env.ELECTRON_IS_DEV || '', 10) === 1;
    const isEnvSet = 'ELECTRON_IS_DEV' in process.env;
    return isEnvSet ? getFromEnv : (process.defaultApp || /node_modules[\\/]electron[\\/]/.test(process.execPath));
})();
/** Wait until electron app is ready (doesn't wait if already ready) */
function waitUntilReady() {
    return new Promise((resolve, reject) => {
        if (electron_1.app.isReady()) {
            resolve();
        }
        else {
            electron_1.app.once('ready', () => { resolve(); });
        }
    });
}
exports.waitUntilReady = waitUntilReady;
//# sourceMappingURL=Util.js.map