"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const events_1 = require("events");
const LogCommon_1 = require("./LogCommon");
/** API for the log used by the renderer process */
class LogRendererApi extends events_1.EventEmitter {
    constructor() {
        super();
        this.entries = [];
        this._nextAddEntryMsgId = 0;
        this.onRefreshEntries = (event, start, entries) => {
            // Add new entries
            for (let i = 0; i < entries.length; i++) {
                this.entries[start + i] = entries[i];
            }
            // Emit event
            this.emit('change', this);
        };
        this.onRemoveEntries = (event, first, last) => {
            // Remove entries
            this.entries.splice(first, last);
            // Emit event
            this.emit('change', this);
        };
    }
    bindListeners() {
        electron_1.ipcRenderer.on(LogCommon_1.LogChannel.refreshEntriesReply, this.onRefreshEntries);
        electron_1.ipcRenderer.on(LogCommon_1.LogChannel.removeEntriesReply, this.onRemoveEntries);
    }
    unbindListeners() {
        electron_1.ipcRenderer.removeListener(LogCommon_1.LogChannel.refreshEntriesReply, this.onRefreshEntries);
        electron_1.ipcRenderer.removeListener(LogCommon_1.LogChannel.removeEntriesReply, this.onRemoveEntries);
    }
    addEntry(preEntry) {
        // Send the entry data (& message id)
        electron_1.ipcRenderer.send(LogCommon_1.LogChannel.addEntry, preEntry);
    }
    addEntryTracked(preEntry) {
        return new Promise((resolve, reject) => {
            // Pick a unique id for this message
            const sentMsgId = this._nextAddEntryMsgId++;
            // Send the entry data (& message id)
            electron_1.ipcRenderer.send(LogCommon_1.LogChannel.addEntry, preEntry, sentMsgId);
            // Add listener for the response
            const listener = (msgId) => {
                if (msgId === sentMsgId) {
                    electron_1.ipcRenderer.removeListener(LogCommon_1.LogChannel.addEntryReply, listener);
                    resolve(sentMsgId);
                }
            };
            electron_1.ipcRenderer.on(LogCommon_1.LogChannel.addEntryReply, listener);
        });
    }
    refreshEntries() {
        electron_1.ipcRenderer.send(LogCommon_1.LogChannel.refreshEntries, 0);
    }
    clearEntries() {
        // (Remove all entries between index 0 and the index of the latest entry)
        electron_1.ipcRenderer.send(LogCommon_1.LogChannel.removeEntries, 0, this.entries.length);
    }
    stringifyEntries() {
        return LogCommon_1.stringifyLogEntries(this.entries);
    }
}
exports.LogRendererApi = LogRendererApi;
//# sourceMappingURL=LogRendererApi.js.map