"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const events_1 = require("events");
const path = require("path");
const Util_1 = require("../Util");
class AppConfigApi extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        /** If this is initialized */
        this._isInit = false;
    }
    get data() {
        if (!this._dataCache) {
            throw new Error('Yo must not access AppConfigApi.data before it has loaded');
        }
        return this._dataCache;
    }
    get fullFlashpointPath() {
        if (this._fullFlashpointPath === undefined) {
            throw new Error('Yo must not access AppConfigApi.data before it has loaded');
        }
        return this._fullFlashpointPath;
    }
    get fullJsonFolderPath() {
        if (this._fullJsonFolderPath === undefined) {
            throw new Error('Yo must not access AppConfigApi.data before it has loaded');
        }
        return this._fullJsonFolderPath;
    }
    /** Initialize (this should be called after construction, and before accessing the data object) */
    async initialize() {
        return new Promise(async () => {
            if (this._isInit) {
                throw new Error('You can only initialize this once');
            }
            // Fetch initial preferences data from main
            const data = await this.fetch();
            // Keep data
            this._dataCache = Util_1.deepCopy(data.data);
            this._fullFlashpointPath = data.fullFlashpointPath;
            this._fullJsonFolderPath = path.posix.join(data.fullFlashpointPath, data.data.jsonFolderPath);
            // Done
            this._isInit = true; // Update Flag
            this.emit('init'); // Emit event
        });
    }
    /** Wait until this is initialized (doesn't wait if already initialized) */
    async waitUtilInitialized() {
        // Check if already initialized
        if (this._isInit) {
            return;
        }
        // Wait for the init event
        await new Promise((resolve) => {
            this.once('init', () => { resolve(); });
        });
    }
    /** Fetch the config data from the main process */
    async fetch() {
        // Send data and wait for response
        return new Promise((resolve, reject) => {
            const data = electron_1.ipcRenderer.sendSync(AppConfigApi.ipcRequestSync);
            if (data) {
                resolve(data);
            }
            else {
                reject(new Error('No data received from config data fetch request'));
            }
        });
    }
}
/** Request Configs Data (renderer -> main) (IPC Event Name) */
AppConfigApi.ipcRequestSync = 'app-configs-api-request-sync';
exports.AppConfigApi = AppConfigApi;
//# sourceMappingURL=AppConfigApi.js.map